package simplelz77compressor;

import java.io.*;
import java.util.zip.*;

/**
* Eine Wrapper-Klasse fr die Deflater- und Inflater-Klassen.
* Komprimiert bzw. dekomprimniert einen Datenstrom nach Huffman.
*/
public class Huffman {
	public static final int BUFFERSIZE = 1024;		// Gre des Puffers
	
	/**
	* Benutzt einen Deflater um die Eingabedaten zu komprimieren
	* @param in Eingabedatenstrom.
	* @param out Ausgabedatenstrom mit den komprimierten Daten.
	* @throws IOException Wenn ein IO-Fehler in den Datenstrmen auftritt.
	*/
	public static void encode( InputStream in, OutputStream out ) throws IOException {
		byte[] inbuffer = new byte[BUFFERSIZE];
		byte[] outbuffer = new byte[BUFFERSIZE];
		
		Deflater compressor = new Deflater( Deflater.BEST_COMPRESSION );
		compressor.setStrategy( Deflater.HUFFMAN_ONLY );
		
		int readamount;
		int compressamount;
		while( (readamount = in.read( inbuffer )) == BUFFERSIZE || in.available() > 0 ){
			compressor.setInput( inbuffer, 0, readamount );
			while( !compressor.needsInput() ){
				compressamount = compressor.deflate( outbuffer );
				out.write( outbuffer, 0, compressamount );
			}
		}
		
		if( readamount < 0 ) readamount = 0;
		
		compressor.setInput( inbuffer, 0, readamount );
		compressor.finish();
		
		while( !compressor.finished() ){
			compressamount = compressor.deflate( outbuffer );
			out.write( outbuffer, 0, compressamount );
		}
		
		compressor.end();
	}
	
	/**
	* Benutzt einen Inflater um die Eingabedaten zu dekomprimieren
	* @param in Eingabedatenstrom.
	* @param out Ausgabedatenstrom mit den dekomprimierten Daten.
	* @throws IOException Wenn ein IO-Fehler in den Datenstrmen auftritt.
	* @throws DataFormatException wenn in "in" keine dekomprimierbaren Daten vorliegen.
	*/
	public static void decode( InputStream in, OutputStream out ) throws IOException, DataFormatException {
		byte[] inbuffer = new byte[BUFFERSIZE];
		byte[] outbuffer = new byte[BUFFERSIZE];
		
		Inflater decompressor = new Inflater();
		
		int amount;
		while( !decompressor.finished() ){
			amount = in.read( inbuffer );
			decompressor.setInput( inbuffer, 0, amount );
			while( !decompressor.needsInput() ){
				amount = decompressor.inflate( outbuffer );
				out.write( outbuffer, 0, amount );
			}
		}
			
		decompressor.end();
	}
}